@@ -1,7 +1,7 @@
 # __init__.py - main module
 # coding: utf-8
 #
-# Copyright (C) 2014 Arthur de Jong
+# Copyright (C) 2014-2015 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -34,12 +34,10 @@ The following prints all keys, decrypting using a password:
 >>> pskc = PSKC('tests/rfc6030-figure7.pskcxml')
 >>> pskc.encryption.derive_key('qwerty')
 >>> for key in pskc.keys:
-...     print key.serial, key.secret
+...     print('%s %s' % (key.serial, str(key.secret.decode())))
 987654321 12345678901234567890
 
-The module should be able to handle most common PSKC files. Checking embedded
-signatures, asymmetric keys and writing files are on the wishlist (patches
-welcome).
+The module should be able to handle most common PSKC files.
 """
 
 
@@ -47,7 +45,7 @@ __all__ = ['PSKC', '__version__']
 
 
 # the version number of the library
-__version__ = '0.2'
+__version__ = '0.3'
 
 
 class PSKC(object):
@@ -72,9 +70,9 @@ class PSKC(object):
         self.mac = MAC(self)
         self.keys = []
         if filename is not None:
-            from pskc.parse import etree
+            from pskc.xml import parse
             try:
-                tree = etree.parse(filename)
+                tree = parse(filename)
             except Exception:
                 raise ParseError('Error parsing XML')
             self.parse(tree.getroot())
@@ -85,7 +83,7 @@ class PSKC(object):
         """Read information from the provided <KeyContainer> tree."""
         from pskc.exceptions import ParseError
         from pskc.key import Key
-        from pskc.parse import find, findall
+        from pskc.xml import find, findall
         if not container.tag.endswith('KeyContainer'):
             raise ParseError('Missing KeyContainer')
         # the version of the PSKC schema
@@ -102,6 +100,14 @@ class PSKC(object):
         for key_package in findall(container, 'pskc:KeyPackage'):
             self.keys.append(Key(self, key_package))
 
+    def make_xml(self):
+        from pskc.xml import mk_elem
+        container = mk_elem('pskc:KeyContainer', Version=self.version,
+                            Id=self.id)
+        for key in self.keys:
+            key.make_xml(container)
+        return container
+
     def add_key(self, **kwargs):
         """Create a new key instance for the PSKC file.
 
@@ -116,3 +122,12 @@ class PSKC(object):
                 raise AttributeError()
             setattr(key, k, v)
         return key
+
+    def write(self, filename):
+        """Write the PSKC file to the provided file."""
+        from pskc.xml import tostring
+        if hasattr(filename, 'write'):
+            filename.write(tostring(self.make_xml()))
+        else:
+            with open(filename, 'wb') as output:
+                self.write(output)
