@@ -1,7 +1,7 @@
 # encryption.py - module for handling encrypted values
 # coding: utf-8
 #
-# Copyright (C) 2014 Arthur de Jong
+# Copyright (C) 2014-2015 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -30,7 +30,7 @@ The encryption key can be derived using the KeyDerivation class.
 
 def unpad(value):
     """Remove padding from the plaintext."""
-    return value[0:-ord(value[-1])]
+    return value[0:-ord(value[-1:])]
 
 
 class EncryptedValue(object):
@@ -51,7 +51,7 @@ class EncryptedValue(object):
 
     def parse(self, encrypted_value):
         """Read encrypted data from the <EncryptedValue> XML tree."""
-        from pskc.parse import find, findbin
+        from pskc.xml import find, findbin
         if encrypted_value is None:
             return
         encryption_method = find(encrypted_value, 'xenc:EncryptionMethod')
@@ -92,14 +92,14 @@ class EncryptedValue(object):
         elif self.algorithm.endswith('#kw-aes128') or \
                 self.algorithm.endswith('#kw-aes192') or \
                 self.algorithm.endswith('#kw-aes256'):
-            from pskc.aeskw import unwrap
+            from pskc.crypto.aeskw import unwrap
             from Crypto.Cipher import AES
             if len(key) * 8 != int(self.algorithm[-3:]) or \
                len(key) not in AES.key_size:
                 raise DecryptionError('Invalid key length')
             return unwrap(self.cipher_value, key)
         elif self.algorithm.endswith('#kw-tripledes'):
-            from pskc.tripledeskw import unwrap
+            from pskc.crypto.tripledeskw import unwrap
             from Crypto.Cipher import DES3
             if len(key) not in DES3.key_size:
                 raise DecryptionError('Invalid key length')
@@ -131,23 +131,25 @@ class KeyDerivation(object):
 
     def parse(self, key_deriviation):
         """Read derivation parameters from a <KeyDerivationMethod> element."""
-        from pskc.parse import find, findint, findbin
+        from pskc.xml import find, findint, findbin
         if key_deriviation is None:
             return
         self.algorithm = key_deriviation.get('Algorithm')
         # PBKDF2 properties
-        pbkdf2 = find(key_deriviation, 'xenc11:PBKDF2-params')
-        if pbkdf2 is None:
-            pbkdf2 = find(key_deriviation, 'pkcs5:PBKDF2-params')
+        pbkdf2 = find(
+            key_deriviation, 'xenc11:PBKDF2-params', 'pkcs5:PBKDF2-params')
         if pbkdf2 is not None:
             # get used salt
-            self.pbkdf2_salt = findbin(pbkdf2, 'Salt/Specified')
+            self.pbkdf2_salt = findbin(
+                pbkdf2, 'Salt/Specified', 'xenc11:Salt/xenc11:Specified')
             # required number of iterations
-            self.pbkdf2_iterations = findint(pbkdf2, 'IterationCount')
+            self.pbkdf2_iterations = findint(
+                pbkdf2, 'IterationCount', 'xenc11:IterationCount')
             # key length
-            self.pbkdf2_key_length = findint(pbkdf2, 'KeyLength')
+            self.pbkdf2_key_length = findint(
+                pbkdf2, 'KeyLength', 'xenc11:KeyLength')
             # pseudorandom function used
-            prf = find(pbkdf2, 'PRF')
+            prf = find(pbkdf2, 'PRF', 'xenc11:PRF')
             if prf is not None:
                 self.pbkdf2_prf = prf.get('Algorithm')
 
@@ -199,7 +201,7 @@ class Encryption(object):
 
     def parse(self, key_info):
         """Read encryption information from the <EncryptionKey> XML tree."""
-        from pskc.parse import find, findall, findtext
+        from pskc.xml import find, findall, findtext
         if key_info is None:
             return
         self.id = key_info.get('Id')
