@@ -1,7 +1,7 @@
 # key.py - module for handling keys from pskc files
 # coding: utf-8
 #
-# Copyright (C) 2014 Arthur de Jong
+# Copyright (C) 2014-2015 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -34,13 +34,13 @@ class DataType(object):
     This class is meant to be subclassed to provide typed access to stored
     values. Instances of this class provide the following attributes:
 
-      plain_value: raw unencrypted value if present (possibly base64 encoded)
+      value: unencrypted value if present
       encrypted_value: reference to an EncryptedValue instance
       value_mac: reference to a ValueMAC instance
     """
 
     def __init__(self, key, element=None):
-        self.plain_value = None
+        self.value = None
         self.encrypted_value = EncryptedValue(key.pskc.encryption)
         self.value_mac = ValueMAC(key.pskc.mac)
         self.parse(element)
@@ -51,16 +51,40 @@ class DataType(object):
         The element is expected to contain <PlainValue>, <EncryptedValue>
         and/or ValueMAC elements that contain information on the actual
         value."""
-        from pskc.parse import find, findtext
+        from pskc.xml import find, findtext
         if element is None:
             return
-        self.plain_value = findtext(element, 'pskc:PlainValue')
+        value = findtext(element, 'pskc:PlainValue')
+        if value is not None:
+            self.value = self.from_text(value)
         self.encrypted_value.parse(find(element, 'pskc:EncryptedValue'))
         self.value_mac.parse(find(element, 'pskc:ValueMAC'))
 
+    def make_xml(self, key, tag):
+        from pskc.xml import find, mk_elem
+        # skip empty values
+        value = self.get_value()
+        if value is None:
+            return
+        # find the data tag and create our tag under it
+        data = find(key, 'pskc:Data')
+        if data is None:
+            data = mk_elem(key, 'pskc:Data', empty=True)
+        element = mk_elem(data, tag, empty=True)
+        mk_elem(element, 'pskc:PlainValue', self.to_text(self.value))
+
+    def get_value(self):
+        """Provide the raw binary value."""
+        if self.value is not None:
+            return self.value
+        if self.encrypted_value.cipher_value:
+            # check MAC and decrypt
+            self.check()
+            return self.from_bin(self.encrypted_value.decrypt())
+
     def set_value(self, value):
-        """Save the plain value."""
-        self.plain_value = self.to_plain(value)
+        """Set the unencrypted value."""
+        self.value = value
         self.encrypted_value.cipher_value = None
 
     def check(self):
@@ -72,45 +96,39 @@ class DataType(object):
 class BinaryDataType(DataType):
     """Subclass of DataType for binary data (e.g. keys)."""
 
-    def get_value(self):
-        """Provide the raw binary value."""
-        # plain value is base64 encoded
-        if self.plain_value is not None:
-            return base64.b64decode(self.plain_value)
-        # check MAC if present
-        self.check()
-        # encrypted value is in correct format
-        return self.encrypted_value.decrypt()
+    def from_text(self, value):
+        """Convert the plain value to native representation."""
+        return base64.b64decode(value)
 
-    def to_plain(self, value):
+    def to_text(self, value):
         """Convert the value to an unencrypted string representation."""
-        if value:
-            return base64.b64encode(value)
+        # force conversion to bytestring on Python 3
+        if not isinstance(value, type(b'')):
+            value = value.encode()
+        return base64.b64encode(value).decode()
+
+    def from_bin(self, value):
+        """Convert the unencrypted binary to native representation."""
+        return value
 
 
 class IntegerDataType(DataType):
     """Subclass of DataType for integer types (e.g. counters)."""
 
-    def get_value(self):
-        """Provide the raw integer value."""
-        # plain value is a string representation of the number
-        if self.plain_value:
-            return int(self.plain_value)
-        # check MAC if present
-        self.check()
-        # decrypted value is big endian encoded
-        value = self.encrypted_value.decrypt()
-        if value is not None:
-            # Python3 has int.from_bytes(value, byteorder='big')
-            v = 0
-            for x in value:
-                v = (v << 8) + ord(x)
-            return v
+    def from_text(self, value):
+        """Convert the plain value to native representation."""
+        return int(value)
 
-    def to_plain(self, value):
+    def to_text(self, value):
         """Convert the value to an unencrypted string representation."""
-        if value not in (None, ''):
-            return str(value)
+        return str(value)
+
+    def from_bin(self, value):
+        """Convert the unencrypted binary to native representation."""
+        result = 0
+        for x in value:
+            result = (result << 8) + ord(x)
+        return result
 
 
 class Key(object):
@@ -197,7 +215,7 @@ class Key(object):
 
     def parse(self, key_package):
         """Read key information from the provided <KeyPackage> tree."""
-        from pskc.parse import find, findtext, findtime, getint, getbool
+        from pskc.xml import find, findtext, findtime, getint, getbool
         if key_package is None:
             return
 
@@ -262,6 +280,60 @@ class Key(object):
 
         self.policy.parse(find(key_package, 'pskc:Key/pskc:Policy'))
 
+    def make_xml(self, container):
+        from pskc.xml import mk_elem
+
+        key_package = mk_elem(container, 'pskc:KeyPackage', empty=True)
+
+        if any(x is not None
+               for x in (self.manufacturer, self.serial, self.model,
+                         self.issue_no, self.device_binding, self.start_date,
+                         self.expiry_date, self.device_userid)):
+            device_info = mk_elem(key_package, 'pskc:DeviceInfo', empty=True)
+            mk_elem(device_info, 'pskc:Manufacturer', self.manufacturer)
+            mk_elem(device_info, 'pskc:SerialNo', self.serial)
+            mk_elem(device_info, 'pskc:Model', self.model)
+            mk_elem(device_info, 'pskc:IssueNo', self.issue_no)
+            mk_elem(device_info, 'pskc:DeviceBinding', self.device_binding)
+            mk_elem(device_info, 'pskc:StartDate', self.start_date)
+            mk_elem(device_info, 'pskc:ExpiryDate', self.expiry_date)
+            mk_elem(device_info, 'pskc:UserId', self.device_userid)
+
+        if self.crypto_module is not None:
+            crypto_module = mk_elem(key_package, 'pskc:CryptoModuleInfo',
+                                    empty=True)
+            mk_elem(crypto_module, 'pskc:Id', self.crypto_module)
+
+        key = mk_elem(key_package, 'pskc:Key', empty=True, Id=self.id,
+                      Algorithm=self.algorithm, )
+        mk_elem(key, 'pskc:Issuer', self.issuer)
+
+        if any((self.algorithm_suite, self.challenge_encoding,
+                self.response_encoding, self.response_length)):
+            parameters = mk_elem(key, 'pskc:AlgorithmParameters', empty=True)
+            mk_elem(parameters, 'pskc:Suite', self.algorithm_suite)
+            mk_elem(parameters, 'pskc:ChallengeFormat',
+                    Encoding=self.challenge_encoding,
+                    Min=self.challenge_min_length,
+                    Max=self.challenge_max_length,
+                    CheckDigits=self.challenge_check)
+            mk_elem(parameters, 'pskc:ResponseFormat',
+                    Encoding=self.response_encoding,
+                    Length=self.response_length,
+                    CheckDigits=self.response_check)
+
+        mk_elem(key, 'pskc:KeyProfileId', self.key_profile)
+        mk_elem(key, 'pskc:KeyReference', self.key_reference)
+        mk_elem(key, 'pskc:FriendlyName', self.friendly_name)
+        self._secret.make_xml(key, 'pskc:Secret')
+        self._counter.make_xml(key, 'pskc:Counter')
+        self._time_offset.make_xml(key, 'pskc:Time')
+        self._time_interval.make_xml(key, 'pskc:TimeInterval')
+        self._time_drift.make_xml(key, 'pskc:TimeDrift')
+        mk_elem(key, 'pskc:UserId', self.key_userid)
+
+        self.policy.make_xml(key)
+
     secret = property(
         fget=lambda self: self._secret.get_value(),
         fset=lambda self, x: self._secret.set_value(x),
