@@ -1,7 +1,7 @@
 # policy.py - module for handling PSKC policy information
 # coding: utf-8
 #
-# Copyright (C) 2014 Arthur de Jong
+# Copyright (C) 2014-2015 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -109,7 +109,7 @@ class Policy(object):
 
     def parse(self, policy):
         """Read key policy information from the provided <Policy> tree."""
-        from pskc.parse import (
+        from pskc.xml import (
             find, findall, findtext, findint, findtime, getint)
         if policy is None:
             return
@@ -136,6 +136,32 @@ class Policy(object):
         # TODO: check if there are other children and make sure
         # policy rejects any key usage (set unknown_policy_elements)
 
+    def make_xml(self, key):
+        from pskc.xml import mk_elem
+        # check if any policy attribute is set
+        if not self.key_usage and all(x is None for x in (
+                self.start_date, self.expiry_date,
+                self.number_of_transactions, self.pin_key_id, self.pin_usage,
+                self.pin_max_failed_attemtps, self.pin_min_length,
+                self.pin_max_length, self.pin_encoding)):
+            return
+        # TODO: raise exception if unknown_policy_elements is set
+
+        policy = mk_elem(key, 'pskc:Policy', empty=True)
+        mk_elem(policy, 'pskc:StartDate', self.start_date)
+        mk_elem(policy, 'pskc:ExpiryDate', self.expiry_date)
+        mk_elem(policy, 'pskc:PINPolicy',
+                PINKeyId=self.pin_key_id,
+                PINUsageMode=self.pin_usage,
+                MaxFailedAttempts=self.pin_max_failed_attemtps,
+                MinLength=self.pin_min_length,
+                MaxLength=self.pin_max_length,
+                PINEncoding=self.pin_encoding)
+        for usage in self.key_usage:
+            mk_elem(policy, 'pskc:KeyUsage', usage)
+        mk_elem(policy, 'pskc:NumberOfTransactions',
+                self.number_of_transactions)
+
     def may_use(self, usage):
         """Check whether the key may be used for the provided purpose."""
         if self.unknown_policy_elements:
@@ -155,4 +181,4 @@ class Policy(object):
         """PIN value referenced by PINKeyId if any."""
         key = self.pin_key
         if key:
-            return key.secret
+            return str(key.secret.decode())
